
#include <stdio.h>
#include <stdlib.h>
#include <config.h>
#include <string.h>

#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <gconf/gconf-client.h>

#include "applet.h"

static void
vmprop_delay_cb(GtkWidget *widget, VmAppletCtl *ctl)
{
    _F();

    ctl->delay = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)) * 1000;
    vm_timer_refresh(ctl);
    panel_applet_gconf_set_int (PANEL_APPLET(ctl->applet), VM_GCONF_DELAY,
                            ctl->delay, NULL);
}

static void
vmprop_mitem_cb(GtkWidget *widget, VmAppletMenuDesc *desc)
{
    VmAppletCtl *ctl = desc->ctl;
    PanelApplet *applet = PANEL_APPLET (ctl->applet);
    gint tmp = GTK_TOGGLE_BUTTON (widget)->active;

    _F();

    if (tmp != desc->enable)
    {
        desc->enable = (gboolean) tmp;
        panel_applet_gconf_set_bool (applet, desc->gconf, desc->enable, NULL);
        vm_popmenu_destroy_items(ctl);
    }
}

static void
vmprop_response_cb(GtkDialog *dialog, gint id, VmAppletCtl *ctl)
{
    _F();

    if (id == GTK_RESPONSE_HELP)
    {
        vm_show_help( ctl, NULL );
        return;
    }
    gtk_widget_destroy (GTK_WIDGET (dialog));
    ctl->propwin = NULL;
}

static gboolean
vmprop_key_writable (PanelApplet *applet, const char *key)
{
    gboolean writable;
    char *fullkey;
    static GConfClient *client = NULL;

    _F();

    if (client == NULL)
        client = gconf_client_get_default ();

    fullkey = panel_applet_gconf_get_full_key (applet, key);
    writable = gconf_client_key_is_writable (client, fullkey, NULL);

    g_free (fullkey);
    return writable;
}

void
vmprop_show(VmAppletCtl *ctl)
{
    GtkWidget *notebook;
    GtkWidget *vbox;
    GtkWidget *frame;
    GtkWidget *widget;
    gboolean sensitive = TRUE;
    gint i;

    _F();

    if (ctl->propwin)
    {
        gtk_window_set_screen (GTK_WINDOW (ctl->propwin),
                   gtk_widget_get_screen (ctl->applet));
        gtk_window_present (GTK_WINDOW (ctl->propwin));
        return;
    }

    ctl->propwin = gtk_dialog_new_with_buttons (_("VM Applet Preferences"), NULL,
                            GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                            NULL);
    gtk_window_set_screen (GTK_WINDOW (ctl->propwin),
                  gtk_widget_get_screen (ctl->applet));

    g_signal_connect (G_OBJECT (ctl->propwin), "response",
          G_CALLBACK (vmprop_response_cb), ctl);

    gtk_dialog_set_default_response (GTK_DIALOG (ctl->propwin), GTK_RESPONSE_CLOSE);
    gtk_dialog_set_has_separator (GTK_DIALOG (ctl->propwin), FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (ctl->propwin), 5);
    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (ctl->propwin)->vbox), 2);
    gtk_window_set_resizable (GTK_WINDOW (ctl->propwin), TRUE);
    gtk_window_set_modal (GTK_WINDOW (ctl->propwin), FALSE);

    notebook = gtk_notebook_new ();
    gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (ctl->propwin)->vbox), notebook,
         TRUE, TRUE, 0);

    /* PAGE: Display  */
    vbox = gtk_vbox_new (FALSE, 18);
    gtk_container_set_border_width (GTK_CONTAINER(vbox), 12);

    widget = gtk_label_new(_("Display"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, widget);

    frame = vm_frame_new(vbox, _("Domain information"));

    /* initialize menu description */
    for (i=0; i < ctl->ndescs; i++)
    {
        VmAppletMenuDesc *desc = &ctl->desc[i];

        sensitive = vmprop_key_writable (PANEL_APPLET (ctl->applet), desc->gconf);
        widget = vm_checkbutton_new(desc->name, desc->enable);

        g_signal_connect(G_OBJECT(widget), "toggled",
                        G_CALLBACK(vmprop_mitem_cb), (gpointer) desc);
        gtk_box_pack_start(GTK_BOX(frame), widget, FALSE, FALSE, 0);
        gtk_widget_set_sensitive(widget, sensitive);
    }

    /* PAGE: Vm  */
    vbox = gtk_vbox_new (FALSE, 18);
    gtk_container_set_border_width (GTK_CONTAINER(vbox), 12);

    widget = gtk_label_new(_("Virtual Domains"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, widget);

    frame = vm_frame_new(vbox, _("Data"));

    /* delay */
    sensitive =  vmprop_key_writable (PANEL_APPLET (ctl->applet), VM_GCONF_DELAY);
    widget = vm_spinbutton_new(1, 3600, 1, ctl->delay / 1000);
    g_signal_connect(G_OBJECT(widget), "value-changed",
                G_CALLBACK(vmprop_delay_cb), (gpointer) ctl);
    vm_prefbox_add(frame, NULL, _("U_pdate every:"), widget, _("seconds"));

    gtk_widget_show_all(ctl->propwin);
}

/*
 * vim: set tabstop=4:
 * vim: set shiftwidth=4:
 * vim: set expandtab:
 */
